#!/bin/ksh

######################################################################
# * Licensed Materials - Property of IBM
# *
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2018 All Rights Reserved   
######################################################################

search="/bin /usr/bin /local/bin /usr/local/bin"
search="$search /sbin /usr/sbin /local/sbin /usr/local/sbin"
function locateCMD {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  inCMD=$1
  if [[ -z "$inCMD" ]]; then
    outCMD=""
  else
    if [[ "$osType" = @(HP-UX|SunOS) ]]; then
      outCMD=$(which $inCMD 2> /dev/null | grep -v "^no $inCMD")
    else
      outCMD=$(which $inCMD 2> /dev/null)
    fi
    if [[ -z "$outCMD" ]]; then
      outCMD=$(find $search -name $inCMD 2> /dev/null)
    fi
  fi
  print $outCMD
}

function Log_report {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  case $1 in
    warn) if [[ -a "$logFile" ]]; then
            print "$PROGNAME warning:  $2, continuing ..." >> $logFile
          fi
          print -u2 "$PROGNAME $WARNING:  $2, $CONTINUING ..."
          sleep 2
          return
          ;;
    fail) if [[ -a "$logFile" ]]; then
            print "$PROGNAME failure:  $2."  >> $logFile
            print -u2 "$(nls_replace 'KCI1200E' "UpdateAutoRun.sh" $logFile)"
          else
            print "$PROGNAME $FAILURE:  $2."
          fi
          exit 2
          ;;
    info) if [[ -a "$logFile" ]]; then
            print "$PROGNAME info:  $2" >> $logFile
          else
            print "$PROGNAME info:  $2"
          fi
          return
          ;;
    infoCont) if [[ -a "$logFile" ]]; then
            print "$PROGblnk     :  $2" >> $logFile
          else
            print "$PROGblnk     :  $2"
          fi
          return
          ;;
    infoRaw) if [[ -a "$logFile" ]]; then
            print "$2" >> $logFile
          else
            print "$2"
          fi
          return
          ;;
  esac
}

function Use_report {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print -u2 "\n$PROGNAME [ -h install_dir ] \n"
  exit 2
}

function set_bootMode {
 #---------------------------------------------------------------------#IV97694
 # Determine whether this system will be booted as systemd or initd    #IV97694
 #---------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
 # pkg-config may not be installed on Ubuntu even when systemd is used #IV97694
  pkgconfigCMD=$(locateCMD pkg-config);                                #IV97694
  if [[ -n "$pkgconfigCMD" ]]; then                                    #IV97694
    $pkgconfigCMD systemd --exists > /dev/null 2>&1;                   #IV97694
    sdExists=$?;                                                       #IV97694
  else                                                                 #IV97694
    sdExists=1;                                                        #IV97694
  fi                                                                   #IV97694
  # Need the systemctl command in the test for Ubuntu                  #IV97694
  systemctlCMD=$(locateCMD systemctl);                                 #IV97694

  if [[ $sdExists -eq 0 ]]; then                                       #IV97694
    boot_mode=systemd;                                                 #IV97694
  else                                                                 #IV97694
    boot_mode=initd;                                                   #IV97694
    if [[ "$linux_dist" = Ubuntu ]]; then                              #IV97694
      if [[ -n "$systemctlCMD" ]]; then
        $systemctlCMD --now > /dev/null 2>&1;                          #IV97694
        if [[ $? -eq 0 ]]; then                                        #IV97694
          boot_mode=systemd;                                           #IV97694
        fi                                                             #IV97694
      fi                                                               #IV97694
    fi                                                                 #IV97694
  fi                                                                   #IV97694

  if [[ "$boot_mode" = systemd ]]; then                                #IV97694
    [[ -z "$systemctlCMD" ]] && 
      Log_report fail "Unable to find systemctl command";              #IV97694
  fi                                                                   #IV97694

  chkconfigCMD=$(locateCMD chkconfig)
  if [[ -z "$chkconfigCMD" ]]; then
    chkconfigCMD=$(locateCMD update-rc.d)
  fi

  if [[ "$boot_mode" = initd && -z "$chkconfigCMD" ]]; then            #IV97694
    Log_report fail "Unable to find chkconfig command";                #IV97694
  fi
}

function set_linuxDist {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  if [[ "$os" = +(li|ls|lx|lp) ]]; then                                #IZ58055
    OS_dist=$(grep '^ID=' /etc/os-release 2> /dev/null | cut -d= -f2 |
              cut -d\" -f2);                                           #IV97694
    # OS_dist from /etc/os-release must be fixed to be the same as     #IV97694
    # linux_dist existing strings in use                               #IV97694
    # os-release: redhat sles ubuntu                                   #IV97694
    # linux_dist: RedHat SUSE Ubuntu                                   #IV97694
    case $OS_dist in                                                   #IV97694
      ubuntu) linux_dist="Ubuntu";                                     #IV97694
        ;;                                                             #IV97694
      rhel)   linux_dist="RedHat";                                     #IV97694
         ;;                                                            #IV97694
      sles)   linux_dist="SUSE";                                       #IV97694
        ;;                                                             #IV97694
      "")     linux_dist="";                                           #IV97694
        ;;                                                             #IV97694
      *)      linux_dist="RedHat";     # Pretend unknown is RedHat     #IV97694
        ;;                                                             #IV97694
    esac                                                               #IV97694
  
    set_bootMode;                                                      #IV97694
  
    if [[ -z "$linux_dist" ]]; then                                    #IV97694
      if [[ $(basename $chkconfigCMD) = "update-rc.d" ]]; then         #IV97694
        linux_dist="Ubuntu";                                           #IV97694
      else                                                             #IV97694
        if [[ "$boot_mode" = systemd ]]; then                          #IV97694
          $systemctlCMD --now > /dev/null 2>&1;                        #IV97694
          if [[ $? -eq 0 ]]; then                                      #IV97694
            linux_dist="RedHat";                                       #IV97694
          else                                                         #IV97694
            linux_dist="SUSE";                                         #IV97694
          fi                                                           #IV97694
        else                                                           #IV97694
          $chkconfigCMD --terse > /dev/null 2>&1                       #IZ58055
          if [[ $? -eq 1 ]]; then                                      #IZ58055
            linux_dist="RedHat";                                       #IZ58055
          else                                                         #IZ58055
            linux_dist="SUSE";                                         #IZ58055
          fi                                                           #IZ58055
        fi                                                             #IV97694
      fi                                                               #IV97694
    fi                                                                 #IV97694
  else                                                                 #IV97694
    # Non-Linux OS uses old style boot scripts                         #IV97694
    boot_mode=initd;                                                   #IV97694
  fi                                                                   #IZ58055
}

function set_rcBase {
 #---------------------------------------------------------------------#IZ58055
 # Initialize rcBaseDir and rcBaseName variables. This is the directory#IZ58055
 # and file name of the init script. The file name is missing the file #IZ58055
 # number that is appended to the end: that comes later.               #IZ58055
 #---------------------------------------------------------------------#IZ58055
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  case "$os" in                                                        #IZ58055
    li|lx|lp|ls)                                                       #IZ58055
      rcBaseDir="/etc/init.d"                                          #IZ58055
      rcBaseName="ITMAgents"                                           #IZ58055
      if [[ "$boot_mode" = systemd ]]; then                            #IV97694
        sdBaseName="ITMAgents"                                         #IV97694
        sdTarget="target"                                              #IV97694
        sdService="service"                                            #IV97694
        if [[ -n "$pkgconfigCMD" ]]; then                              #IV97694
          pkgConfigOPT="systemd --variable=systemdsystemunitdir"       #IV97694
          sdBaseDir="$($pkgconfigCMD $pkgConfigOPT 2> /dev/null)";     #IV97694
        else                                                           #IV97694
          if [[ "$linux_dist" = Ubuntu ]]; then                        #IV97694
            sdBaseDir="/lib/systemd/system";                           #IV97694
          else                                                         #IV97694
            sdBaseDir="/usr/lib/systemd/system";                       #IV97694
          fi                                                           #IV97694
        fi                                                             #IV97694
      fi                                                               #IV97694
      ;;                                                               #IZ58055
    hp)                                                                #IZ58055
      rcBaseDir="/sbin/init.d"                                         #IZ58055
      rcBaseName="ITMAgents"                                           #IZ58055
      ;;                                                               #IZ58055
    so)                                                                #IZ58055
      rcBaseDir="/etc/init.d"                                          #IZ58055
      rcBaseName="ITMAgents"                                           #IZ58055
      ;;                                                               #IZ58055
    ai)                                                                #IZ58055
      rcBaseDir="/etc"                                                 #IZ58055
      rcBaseName="rc.itm"                                              #IZ58055
      ;;                                                               #IZ58055
    *)                                                                 #IZ58055
      Log_report fail "$os platform is not supported."                 #IZ58055
      ;;                                                               #IZ58055
  esac                                                                 #IZ58055
}                                                 # (set_rcBase())     #IZ58055
                                                                       #IZ58055
function set_filenum {
 #---------------------------------------------------------------------#IZ58055
 # Initialize the filenum and first_time variables. The filenum        #IZ58055
 # variable represents the file number that is appended to the rcBase  #IZ58055
 # variable to come up with the complete file name in rcFile. The      #IZ58055
 # first_time variable indicates whether this is a new file number (1) #IZ58055
 # which means this is the first time an init script has been created. #IZ58055
 #---------------------------------------------------------------------#IZ58055
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  filenum=$(cat $CANDLEHOME/registry/AutoStart 2> /dev/null);          #IZ58055
  if [[ $filenum -eq 0 ]]; then                                        #IZ58055
    # begin feature 204833 (RFE 33478)
    # When -s switch for install.sh is used during pristine install we
    # skip updating startup scripts.
    # In this case value 0 is stored in registry/AutoStart file.
    # All subsequent installations/upgrades will skip startup script
    # update. In order to restore startup script update functionality
    # it's enough to remove registry/AutoStart file.
    if [[ -f "$CANDLEHOME/registry/AutoStart" ]]; then
      #$NLS-E$ "Startup script update has been skipped."
      nls_replace 'KCI1365I';
      exit 0;
    fi
    # end feature 204833 (RFE 33478)
    first_time=1                                 # first time          #IZ58055
    filenum=1;                                                         #IZ58055
    rcTestFile="$rcBaseDir/$rcBaseName"
    if [[ "$boot_mode" = initd ]]; then
      sdTestFile="$rcTestFile"
      sdTestSuffix=""
    else
      sdTestFile="$sdBaseDir/$sdBaseName"
      sdTestSuffix=".$sdTarget"
    fi
    while [[ -f "${rcTestFile}${filenum}" ||
             -f "${sdTestFile}${filenum}${sdTestSuffix}" ]]; do        #IZ58055
      filenum=$(( $filenum + 1 ))                                      #IZ58055
    done                                                               #IZ58055
    print $filenum > $CANDLEHOME/registry/AutoStart                    #IZ58055
  else                                                                 #IZ58055
    first_time=0                                 # not first time      #IZ58055
  fi                                                                   #IZ58055
}                                                # (set_filenum())     #IZ58055

function remove_initd_script_for_systemd {
  #--------------------------------------------------------------------#IV97694
  # Define the commands to undo autostart for our platform.            #IV97694
  #--------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  case "$os" in                                                        #IV97694
    li|lx|lp|ls)                                                       #IV97694
      cmd="chkconfig --del ITMAgents$filenum > /dev/null 2>&1"         #IV97694
      cmd="$cmd;rm -f /etc/init.d/ITMAgents$filenum"                   #IV97694
      cmd="$cmd;rm -f /etc/rc.d/rc[0-6].d/K10ITMAgents$filenum"        #IV97694
      cmd="$cmd;rm -f /etc/rc.d/rc[0-6].d/S99ITMAgents$filenum"        #IV97694
      cmd="$cmd;rm -f /etc/init.d/rc[0-6].d/K10ITMAgents$filenum"      #IV97694
      cmd="$cmd;rm -f /etc/init.d/rc[0-6].d/S99ITMAgents$filenum"      #IV97694
      cmd="$cmd;rm -f /var/lock/subsys/ITMAgents$filenum 2> /dev/null" #IV97694
      ;;                                                               #IV97694
    *)                         # Don't do this if this is not linux... #IV97694
      cmd=""                                                           #IV97694
      ;;                                                               #IV97694
  esac                                           # (case(os))          #IV97694
  eval "$cmd"                                                          #IV97694
}                                                                      #IV97694

function create_initd_script {
 #---------------------------------------------------------------------#IZ58055
 # Create a new init script. This base script is later modified with   #IZ58055
 # agent-specific information.                                         #IZ58055
 #---------------------------------------------------------------------#IZ58055
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  if [[ "$os" = "so" ]]; then                                          #IZ58055
    print "#!/bin/sh"                                           > "$1" #IZ58055
  else                                                                 #IZ58055
    print "#!/bin/ksh"                                          > "$1" #IZ58055
  fi                                                                   #IZ58055
                                                                       #IZ58055
  print "# (C) Copyright IBM Corporation 2007, 2012."          >> "$1" #IV23322
  print "# All Rights Reserved"                                >> "$1" #IZ58055
  print "# IBM Tivoli Monitoring"                              >> "$1" #IZ58055
                                                                       #IZ58055
  #--------------------------------------------------------------------#IZ58055
  # For Linux, we need to add a block of entries that define our needs #IZ58055
  # to chkconfig. The format is different for RedHat vs. SUSE.         #IZ58055
  #--------------------------------------------------------------------#IZ58055
  if [[ "$os" = +(li|ls|lx|lp) ]]; then                                #IZ58055
    print "#"                                                  >> "$1" #IZ58055
    print "### BEGIN INIT INFO"                                >> "$1" #IZ58055
    if [[ "$linux_dist" = "RedHat" ]]; then      # RedHat...           #IZ58055
      print "# chkconfig: 2345 99 10"                          >> "$1" #IZ58055
      print "# description: IBM Tivoli Monitoring"             >> "$1" #IZ58055
    else                                         # SUSE...             #IZ58055
      print "# Provides:          ITMAgents$filenum"           >> "$1" #IZ58055
      print "# Required-Start:    \$all"                       >> "$1" #IZ58055
      print "# Required-Stop:"                                 >> "$1" #IZ58055
      print "# Default-Start:     2 3 4 5"                     >> "$1" #IZ58055
      print "# Default-Stop:      0 6"                         >> "$1" #IZ58055
      print "# Short-Description: IBM Tivoli Monitoring"       >> "$1" #IZ58055
      print "# Description:       IBM Tivoli Monitoring"       >> "$1" #IZ58055
    fi                                                                 #IZ58055
    print "### END INIT INFO"                                  >> "$1" #IZ58055
  fi                                                                   #IZ58055
                                                                       #IZ58055
  #--------------------------------------------------------------------#IZ58055
  # Finally add the code portion of the script. We pull in the content #IV23322
  # of ITMAgents.sh to use as a basis. This gets updated later.        #IV23322
  #--------------------------------------------------------------------#IZ58055
  print "#"                                                    >> "$1" #IV23322
  print "CANDLEHOME=\"$CANDLEHOME\""                           >> "$1" #IV23322
  print "export CANDLEHOME"                                    >> "$1" #IV23322
  cat $CANDLEHOME/bin/ITMAgents.sh                             >> "$1" #IV23322
                                                                       #IZ58055
  chmod 755 "$1"                                 # set execute perms   #IZ58055
}                                        # (create_initd_script())     #IZ58055

function create_target_script {
 #---------------------------------------------------------------------#IV97694
 # Create a new systemd target script.                                 #IV97694
 # This base script is later modified with agent-specific information. #IV97694
 #---------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing

  # Systemd Target File Header                                         #IV97694
  cat << EOF > "$1"
[Unit]
Description=IBM Tivoli Monitoring for "$CANDLEHOME"
Requires=multi-user.target
After=multi-user.target
Conflicts=shutdown.target
[Install]
EOF
  chmod 644 "$1"                                 # set execute perms   #IV97694
}                                        # (create_target_script())    #IV97694

function create_service_script {
 #---------------------------------------------------------------------#IV97694
 # Create a new systemd service script.                                #IV97694
 # This script is created with agent-specific information.             #IV97694
 #---------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing

  eval typeset USER=$2                                                 #IV97694
  eval typeset PP=$3                                                   #IV97694
  eval typeset INST=$4                                                 #IV97694
  eval typeset AUTOSTART=$5                                            #IV97694
  # Default time-out is 15 minutes. 5 minutes longer than ITM time-out #IV97694
  typeset TIMEOUT="TimeoutSec=900";                                    #IV97694

  getCmdWrappersSystemd $PP

  # itmcmd server start/stop TEMSNAME
  # itmcmd agent -p INST start/stop um
  # itmcmd agent -o INST start/stop PP (except um)
  if [[ "$PP" = "ms" ]]; then 
    PP=$INST
    INST=""
  elif [[ -n "$INST" ]]; then
    if [[ "$PP" = "um" ]]; then
      INST=" -p $INST"
    else
      INST=" -o $INST"
    fi
  fi

  # Systemd Service File Header                                        #IV97694
  cat << EOF > "$1"
[Unit]
Description=IBM Tivoli Monitoring for "$CANDLEHOME"
After=multi-user.target
Before=shutdown.target
Conflicts=shutdown.target
[Service]
Type=forking
#$linux_dist
$TIMEOUT
KillMode=none
GuessMainPID=no
RemainAfterExit=yes
Restart=no
Environment=CANDLEHOME=$CANDLEHOME
Environment=ITMsystemctl=yes
User=$USER
#=============================== IMPORTANT NOTE ===================#
# No user modifications will be preserved in this file.            #
# User ID overrides and activation of ExecStart/ExecStop commands  #
# must be made in the file config/kcirunas.cfg to be preserved.    #
#=============================== IMPORTANT NOTE ===================#
ExecStart=${CmdWrapFront}${INST} start $PP $CmdWrapBack
ExecStop=${CmdWrapFront}${INST} stop $PP $CmdWrapBack
[Install]
WantedBy=multi-user.target
EOF

  chmod 644 "$1"                                 # set execute perms   #IV97694

  if [[ "$PP" = "cj" ]]; then                    # do not start cj
    tempCJ=$(cat $1)
    message="/bin/ksh -c \"print Do not autostart cj component\""
    print "$tempCJ" |
      sed 's@^\(ExecStart=\).*$@\1'"$message"'@' > $1
  fi 

}                                        # (create_service_script())   #IV97694

function set_runlevels {
 #---------------------------------------------------------------------#IZ58055
 # Tell the operating system about our init script.                    #IZ58055
 #---------------------------------------------------------------------#IZ58055
                                                                       #IZ58055
 #---------------------------------------------------------------------#IZ58055
 # For LINUX systems, we used to create symbolic links manually. Since #IZ58055
 # SUSE requires chkconfig to manage runlevels and RedHat supports it, #IZ58055
 # we switched over to that instead. However, we have to clean up the  #IZ58055
 # old links that may have predated our use of chkconfig.              #IZ58055
 #---------------------------------------------------------------------#IZ58055
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  if [[ "$os" = +(li|ls|lx|lp) ]]; then                                #IZ58055
    if [[ "$boot_mode" = initd ]]; then                                #IV97694
      runName="$rcBaseName$filenum"                                    #IZ58055
      if [[ $first_time -ne 1 ]]; then          # need to rm old links #IZ58055
        if [[ "$linux_dist" = "RedHat" ]]; then  # RedHat...           #IZ58055
          rm -f /etc/rc.d/rc[0-6].d/K10$runName                        #IZ58055
          rm -f /etc/rc.d/rc[0-6].d/S99$runName                        #IZ58055
        elif [[ "$linux_dist" = "SUSE" ]]; then  # SUSE...
          rm -f /etc/init.d/rc[0-6].d/K10$runName                      #IZ58055
          rm -f /etc/init.d/rc[0-6].d/S99$runName                      #IZ58055
        else                                     # Ubuntu...
          $chkconfigCMD $runName remove
        fi                                                             #IZ58055
      fi                                                               #IZ58055
      if [[ "$linux_dist" != "Ubuntu" ]]; then
        $chkconfigCMD --del "$runName" > /dev/null 2>&1                #IZ58055
        $chkconfigCMD --add "$runName" > /dev/null                     #IZ58055
      fi
      if [[ "$linux_dist" = "RedHat" ]]; then    # RedHat...           #IZ58055
        $chkconfigCMD --levels 2345 "$runName" on > /dev/null          #IZ58055
      elif [[ "$linux_dist" = "SUSE" ]]; then    # SUSE...             #IZ58055
        $chkconfigCMD --set "$runName" 2345 > /dev/null                #IZ58055
      else                                     # Ubuntu...
        $chkconfigCMD $runName defaults
      fi                                                               #IZ58055
    else                                                               #IV97694
      # Reload systemd since existing files may have changed           #IV97694
      $systemctlCMD daemon-reload;                                     #IV97694
    fi                                                                 #IV97694
  fi                                                                   #IZ58055
                                                                       #IZ58055
  #--------------------------------------------------------------------#IZ58055
  # For HP systems, we just create symbolic links.                     #IZ58055
  #--------------------------------------------------------------------#IZ58055
  if [[ "$os" = hp ]]; then                                            #IZ58055
    runName="$rcBaseName$filenum"                                      #IZ58055
                                                                       #IZ58055
    ln -fs "$rcFile" "/sbin/rc0.d/K100$runName"                        #IZ58055
    ln -fs "$rcFile" "/sbin/rc1.d/K100$runName"                        #IZ58055
    ln -fs "$rcFile" "/sbin/rc2.d/S500$runName"                        #IZ58055
  fi                                                                   #IZ58055
                                                                       #IZ58055
  #--------------------------------------------------------------------#IZ58055
  # For Solaris systems, we create symbolic links. For some reason,    #IZ58055
  # Solaris does not seem to always respect the -f option, so we need  #IZ58055
  # to redirect stderr to prevent messages from showing to the user.   #IZ58055
  #--------------------------------------------------------------------#IZ58055
  if [[ "$os" = so ]]; then                                            #IZ58055
    runName="$rcBaseName$filenum"                                      #IZ58055
                                                                       #IZ58055
    ln -fs "$rcFile" "/etc/rc0.d/K10$runName" 2> /dev/null             #IZ58055
    ln -fs "$rcFile" "/etc/rc1.d/K10$runName" 2> /dev/null             #IZ58055
    ln -fs "$rcFile" "/etc/rc2.d/S99$runName" 2> /dev/null             #IZ58055
  fi                                                                   #IZ58055
                                                                       #IZ58055
  #--------------------------------------------------------------------#IZ58055
  # For AIX systems, we use mkitab to create the entry we need in      #IZ58055
  # /etc/inittab. We make a backup the first time we are installed.    #IZ58055
  #--------------------------------------------------------------------#IZ58055
  if [[ "$os" = ai ]]; then                                            #IZ58055
    runName="rcitm$filenum"                                            #IZ58055
                                                                       #IZ58055
    if [[ $first_time -eq 1 ]]; then                                   #IZ58055
      cp -p /etc/inittab /etc/inittab.itm$filenum                      #IZ58055
    fi                                                                 #IZ58055
                                                                       #IZ58055
    rmitab "$runName" 2> /dev/null            # clean up old entry     #IZ58055
                                                                       #IZ58055
    tab_opt=""                                                         #IZ58055
    lsitab rcnfs > /dev/null 2>&1             # check for rcnfs        #IZ58055
    if [[ $? -eq 0 ]]; then                   # if it's there...       #IZ58055
      tab_opt="-i rcnfs"                      # rcitm is after rcnfs   #IZ58055
    else                                                               #IZ58055
      lsitab rctcpip > /dev/null 2>&1         # check for rctcpip      #IZ58055
      if [[ $? -eq 0 ]]; then                 # if it's there...       #IZ58055
        tab_opt="-i rctcpip"                  # rcitm is after rctcpip #IZ58055
      fi                                                               #IZ58055
    fi                                                                 #IZ58055
                                                                       #IZ58055
    mkitab $tab_opt "$runName:2:once:$rcFile start > /dev/console 2>&1" #IV23322
  fi                                                                   #IZ58055
}                                              # (set_runlevels())     #IZ58055

#This is to get instance from env file.
function getInstance {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  pCode=$1
  instance=""

  defaultUser=$(getDefaultUser $pCode)
  defaultAutoStart=$(getDefaultAutoStart $pCode)
  [[ -z "$defaultAutoStart" ]] && defaultAutoStart=yes;                          #IV97694

  [[ -n "$defaultUser" ]] && user="$defaultUser"
# For Oracle and Sybase we will check <hostname>_<pp>_<instance>.cfg file
  if [[ "$pCode" = @(or|oy|mq) ]]; then
    host=$(hostname | cut -d. -f1)
    cfgPref="$CANDLEHOME/config/$host"
    for rec in $(
      ls "$cfgPref_$pCode_*.cfg" "$cfgPref\.*_$pCode_*.cfg" 2>/dev/null |
      egrep -v -e 'kcirunas|kdyrunas') ; do
      rec=${rec##*/}  # Remove the directory path.
      rec=${rec%.*}   # Remove the ".cfg" extension.
      if [[ "$rec" = "${host}."* ]]; then                              #IZ58877
        instance=${rec#${host}.*_${pCode}_};                           #IZ58877
      else                                                             #IZ58877
        instance=${rec#${host}_${pCode}_};                             #IZ58877
      fi;                                                              #IZ58877
      [[ -z "$instance" ]] && continue
      if [[ "$boot_mode" = initd ]]; then                              #IV97694
        isExist=$(grepInStart "agent -o $instance start $pCode");      #IV33132
        [[ -z "$isExist" ]] &&
          writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
      else
        # Do not replace <instance> <autostart> entries with 
        # <default> <autostart> entries                                #IV97694
        isExist=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"$defaultAutoStart\""); #IV97694
        [[ -z "$isExist" ]] && 
          writeComponentInstance $user $pCode $instance $defaultAutoStart; #IV97694
      fi
    done
    if [[ "$pCode" != "mq" ]]; then
      return
    fi
  fi

# the instance is saved as platform for certain agents
  instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
  instanceFile=$(print "$instanceFile" | grep -v ^default |
                 grep -v ^$os | grep -v FIELDSEP)
  instanceFile=$(print "$instanceFile" | cut -d"|" -f1 | sort -u)
  # defect 73338 - use -p for um instances
  if [[ "$pCode" = "um" ]]; then
    parm="-p"
  else
    parm="-o"
  fi
  if [[ -n "$instanceFile" ]]; then
    for instance in $instanceFile ; do
      if [[ "$boot_mode" = initd ]]; then                              #IV97694
        isExist=$(grepInStart "agent $parm $instance start $pCode");   #IV33132
        [[ -z "$isExist" ]] &&
          writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
      else
        # Do not replace <instance> <autostart> entries with 
        # <default> <autostart> entries                                #IV97694
        isExist=$(grepInComponentInstance "\"$pCode\" \"$instance\""); #IV97694
        [[ -z "$isExist" ]] && 
          writeComponentInstance $user $pCode $instance $defaultAutoStart; #IV97694
      fi
    done
  else

# Now let's check for env files.
#Some instances are saved as variables in the env file
    instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
    instanceFile=$(print "$instanceFile" | grep -v ^default | grep -w INSTANCE)
    for instanceLine in $instanceFile ; do
      instance=$(print "$instanceLine"| cut -d"|" -f3)
      [[ -z "$instance" ]] && continue
      if [[ "$boot_mode" = initd ]]; then                              #IV97694
        isExist=$(grepInStart "agent $parm $instance start $pCode");   #IV33132
        [[ -z "$isExist" ]] &&
          writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
      else
        # When adding an instance record,                              #IV97694
        # we have to remove the !autostart record if it exists         #IV97694
        isExistNo=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"no\"");              #IV97694
        isExistYes=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"yes\"");            #IV97694
        isExist=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"$defaultAutoStart\""); #IV97694
        case "$defaultAutoStart" in                                    #IV97694
          yes)                                                         #IV97694
            [[ -n "$isExistNo" ]] &&
              deleteComponentInstance $user $pCode $instance no;       #IV97694
          ;;                                                           #IV97694
          no)                                                          #IV97694
            [[ -n "$isExistYes" ]] &&
              deleteComponentInstance $user $pCode $instance yes;      #IV97694
          ;;                                                           #IV97694
        esac                                                           #IV97694
        [[ -z "$isExist" ]] && 
          writeComponentInstance $user $pCode $instance $defaultAutoStart; #IV97694
      fi
     done
  fi

  if [[ -f $CANDLEHOME/bin/.agent.$pCode ]]; then
    prdlIst=$pCode

    cur_dir=$(pwd)
    cd $CANDLEHOME/config

    for inst in $(ls $pCode_*.environment 2> /dev/null) ; do
      instname=${inst#${pCode}_}
      instname=${instname%.environment}
      if [[ "$boot_mode" = initd ]]; then                              #IV97694
        isExist=$(grepInStart "agent $parm $instance start $pCode");   #IV33132
        [[ -z "$isExist" ]] &&
          writeAgentInstanceToStartStop $user $pCode $instance $defaultAutoStart
      else
        # When adding an instance record,                              #IV97694
        # we have to remove the !autostart record if it exists         #IV97694
        isExistNo=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"no\"");              #IV97694
        isExistYes=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"yes\"");            #IV97694
        isExist=$(grepInComponentInstance "\"$pCode\" \"$instance\" \"$defaultAutoStart\""); #IV97694
        case "$defaultAutoStart" in                                    #IV97694
          yes)                                                         #IV97694
            [[ -n "$isExistNo" ]] &&
              deleteComponentInstance $user $pCode $instance no;       #IV97694
          ;;                                                           #IV97694
          no)                                                          #IV97694
            [[ -n "$isExistYes" ]] &&
              deleteComponentInstance $user $pCode $instance yes;      #IV97694
          ;;                                                           #IV97694
        esac                                                           #IV97694
        [[ -z "$isExist" ]] && 
          writeComponentInstance $user $pCode $instance $defaultAutoStart; #IV97694
      fi
    done
      if [[ "$boot_mode" = initd ]]; then                              #IV97694
        isExist=$(grepInStart "gent start $pCode");                    #IV33132
        [[ -z "$isExist" ]] &&
          writeAgentToStartStop $user $pCode $defaultAutoStart         #IV33132
      else
        # When adding a non-instance record,                           #IV97694
        # we have to remove the !autostart record if it exists         #IV97694
        isExistNo=$(grepInComponentInstance "\"$pCode\" \"\" \"no\"");              #IV97694
        isExistYes=$(grepInComponentInstance "\"$pCode\" \"\" \"yes\"");            #IV97694
        isExist=$(grepInComponentInstance "\"$pCode\" \"\" \"$defaultAutoStart\""); #IV97694
        case "$defaultAutoStart" in                                    #IV97694
          yes)                                                         #IV97694
            [[ -n "$isExistNo" ]] &&
              deleteComponentInstance $user $pCode "" no;              #IV97694
          ;;                                                           #IV97694
          no)                                                          #IV97694
            [[ -n "$isExistYes" ]] &&
              deleteComponentInstance $user $pCode "" yes;             #IV97694
          ;;                                                           #IV97694
        esac                                                           #IV97694
        [[ -z "$isExist" ]] && 
          writeComponentInstance $user $pCode "" $defaultAutoStart;    #IV97694
      fi

    cd $cur_dir
  fi

}

function getCmdWrappersInitd {
  #--------------------------------------------------------------------#IZ68336
  # Set strings that wrap around the commands to be launched in the    #IZ68336
  # start/stop script:                                                 #IZ68336
  # - For z/OS systems, we need to launch 32-bit TEMS and TEPS in      #IZ68336
  #   32-bit mode using the s390 command.                              #IZ68336
  # - For all systems, we need to launch under ksh because some other  #IZ68336
  #   shells (csh and tcsh, specifically) don't support redirection of #IZ68336
  #   stderr.                                                          #IZ68336
  # - For all systems, we run $CANDLEHOME/bin/itmcmd and redirect both #IZ68336
  #   stdout and stderr to /dev/null.                                  #IZ68336
  # We do this by setting variables "CmdWrapFront" and "CmdWrapBack".  #IZ68336
  # The command to be launched is enclosed between these to accomplish #IZ68336
  # the above items.                                                   #IZ68336
  #--------------------------------------------------------------------#IZ68336
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  pcCode=$1                                                            #IZ68336
  s390cmd=""                                                           #IZ68336
  #--------------------------------------------------------------------#IZ68336
  # Check to see if                                                    #IZ68336
  # a) we are setting up a 32-bit TEMS or TEPS and                     #IZ68336
  # b) if we are on a a 64-bit zLinux system.                          #IZ68336
  # If so, then we need to start the TEMS or TEPS in 32-bit mode using #IZ68336
  # the s390 command.                                                  #IZ68336
  #--------------------------------------------------------------------#IZ68336
  if [[ "$cArch" = ls3* && "$pcCode" = +(cq|ms) ]]; then
    if [[ -z $(find $CANDLEHOME/registry/ -name ${pcCode}ls3*[5-6].ver) ]]; then
      if [[ -z "$realMode" ]]; then            # if first time...      #IZ68336
        realMode=$(
          $CANDLEHOME/bin/checkZMode.sh|grep realMode|cut -d":" -f2)   #IZ68336
      fi                                                               #IZ68336
      if [[ "$realMode" != "32" ]]; then                               #IZ68336
        s390cmd="s390 "                                                #IZ68336
      fi                                                               #IZ68336
    fi                                                                 #IZ68336
  fi                                                                   #IZ68336
                                                                       #IZ68336
  #--------------------------------------------------------------------#IZ68336
  # For all systems, we need to force the startup command to run under #IZ68336
  # ksh. This takes care of problems when root's default shell is csh  #IZ68336
  # or tcsh.                                                           #IZ68336
  #--------------------------------------------------------------------#IZ68336
  CmdWrapSearch="$CANDLEHOME/bin/itmcmd";                              #IV33132
  CmdWrapFront="/bin/ksh -c '${s390cmd}${CmdWrapSearch}";              #IV33132
  CmdWrapBack=">/dev/null 2>&1'";                                      #IZ68336
}                                           # (getCmdWrappersInitd())  #IZ68336

function getCmdWrappersSystemd {
  #--------------------------------------------------------------------#IZ68336
  # Set strings that wrap around the commands to be launched in the    #IZ68336
  # systemd service script:                                            #IZ68336
  # - For z/OS systems, we need to launch 32-bit TEMS and TEPS in      #IZ68336
  #   32-bit mode using the s390 command.                              #IZ68336
  # - For all systems, we need to launch under ksh because some other  #IZ68336
  #   shells (csh and tcsh, specifically) don't support redirection of #IZ68336
  #   stderr.                                                          #IZ68336
  # - For all systems, we run $CANDLEHOME/bin/itmcmd.                  #IZ68336
  # We do this by setting variables "CmdWrapFront" and "CmdWrapBack".  #IZ68336
  # The command to be launched is enclosed between these to accomplish #IZ68336
  # the above items.                                                   #IZ68336
  #--------------------------------------------------------------------#IZ68336
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  pcCode=$1                                                            #IZ68336
  s390cmd=""                                                           #IZ68336
  #--------------------------------------------------------------------#IZ68336
  # If we are setting up a 32-bit TEMS or TEPS and we are on a 64-bit  #IZ68336
  # zLinux system, then we need to start the TEMS or TEPS in 32-bit    #IZ68336
  # mode using the s390 command.                                       #IZ68336
  #--------------------------------------------------------------------#IZ68336
  if [[ "$cArch" = ls3* && "$pcCode" = +(cq|ms) ]]; then
    if [[ -z $(find $CANDLEHOME/registry/ -name ${pcCode}ls3*[5-6].ver) ]]; then
      if [[ -z "$realMode" ]]; then            # if first time...      #IZ68336
        realMode=$(
          $CANDLEHOME/bin/checkZMode.sh|grep realMode|cut -d":" -f2)   #IZ68336
      fi                                                               #IZ68336
      [[ "$realMode" != "32" ]] && s390cmd="s390 "                     #IZ68336
    fi                                                                 #IZ68336
  fi                                                                   #IZ68336

  #--------------------------------------------------------------------#IZ68336
  # For all systems, we need to force the startup command to run under #IZ68336
  # ksh. This takes care of problems when root's default shell is csh  #IZ68336
  # or tcsh.                                                           #IZ68336
  #--------------------------------------------------------------------#IZ68336
  if [[ "$pcCode" = ms ]]; then
    CmdWrapSearch="$CANDLEHOME/bin/itmcmd server";                     #IV33132
  else
    CmdWrapSearch="$CANDLEHOME/bin/itmcmd agent";                      #IV33132
  fi                                                                   #IZ68336
#  CmdWrapFront="/bin/ksh -c '${s390cmd}${CmdWrapSearch}";             #IV33132
#  CmdWrapBack=">/dev/null 2>&1'";                                     #IZ68336
  CmdWrapFront="${s390cmd}${CmdWrapSearch}";                           #IV33132
  CmdWrapBack="";                                                      #IZ68336
}                                         # (getCmdWrappersSystemd())  #IZ68336

function writeCmdToStartStop {
  #--------------------------------------------------------------------#IV33132
  # Write a single autostart command to the start or stop file.        #IV33132
  #                                                                    #IV33132
  # - If kxxrunas specifies not to autostart or if the existing entry  #IV33132
  #   is commented out, the new entry is commented out.                #IV33132
  #                                                                    #IV33132
  # - If kxxrunas specifies a userid of "DO_NOT_SU", the new entry is  #IV33132
  #   generated without the su wrapper.                                #IV33132
  #                                                                    #IV33132
  # Some commands change over time, so we have a dual search mechanism #IV33132
  # to check for comments. First, we look for the current              #IV33132
  # implementation. If we don't find it, we look for a set of          #IV33132
  # commands passed to us. These are separated by the ":" character in #IV33132
  # the variable $isrch.                                               #IV33132
  #--------------------------------------------------------------------#IV33132
  eval $(itmsh_trace_func "${0}" "${@}");    # enable function tracing
  typeset icmd="$1";                         # command to add          #IV33132
  typeset isrch="$2";                        # old commands to search  #IV33132
  typeset iuid="$3";                         # su userid (kcirunas)    #IV33132
  typeset iautostart="$4";                   # autostart (kcirunas)    #IV33132
  typeset ifile="$5";                        # start/stop file         #IV33132
                                                                       #IV33132
  typeset commentMode="   ";                   # assume active entries #IV33132
  if [[ "$iautostart" = "no" ]]; then        # kcirunas says no start  #IV33132
    commentMode="#  ";                         # comment it            #IV33132
  else                                         # chk commented entries #IV33132
    $grepCMD -q "^[ \t]*#.*${CmdWrapSearch} $icmd" "$backRcFile" 2>/dev/null; #IV33132.1
                                                                       #IV33132
     if [[ $? -eq 0 ]]; then                 # found commented record  #IV33132
       commentMode="#  ";                      # keep it commented     #IV33132
     elif [[ -n "$isrch" ]]; then                                      #IV33132
       typeset isrch1;                         # one cmd from $isrch   #IV33132
       print "$isrch" | tr ':' '\n' | while read isrch1; do            #IV33132
          $grepCMD -q "^[ \t]*#.*${CmdWrapSearch} $isrch1" "$backRcFile" 2>/dev/null; #IV33132.1
          if [[ $? -eq 0 ]]; then             # found commented record #IV33132
            commentMode="#  ";                # keep it commented      #IV33132
          fi;                                 # (if($?-eq0))           #IV33132
        done;                                 # (while(read...))       #IV33132
     fi;                                      # (else($?-eq0))         #IV33132
  fi;                                         # (else($iauutostar...)) #IV33132
                                                                       #IV33132
  if [[ "$iuid" = "DO_NOT_SU" ]]; then        # su disabled?           #IV33132
    typeset suWrapFront="";                                            #IV33132
    typeset suWrapBack="";                                             #IV33132
  else                                                                 #IV33132
    typeset suWrapFront="$su_value - $iuid -c \"";                     #IV33132
    typeset suWrapBack="\"";                                           #IV33132
  fi;                                           # (else($iuid="DO...)) #IV33132
                                                                       #IV33132
  print -- "${commentMode}${suWrapFront}${CmdWrapFront} $icmd ${CmdWrapBack}${suWrapBack}" >> "$ifile"; #IV33132
                                                                       #IV33132
}                                            # (writeCmdToStartStop()) #IV33132

######################################################################
######################################################################

function writeAgentInstanceToStartStop {
  #--------------------------------------------------------------------#IV33132
  # $1 = su userid from runas file                                     #IV33132
  # $2 = product code                                                  #IV33132
  # $3 = instance                                                      #IV33132
  # $4 = autostart (yes|no) from xxxrunas.cfg... optional              #IV33132
  #--------------------------------------------------------------------#IV33132
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  getCmdWrappersInitd "$2";                                            #IV33132

  # defect 73338 - use -p for um instances
  if [[ "$2" = "um" ]]; then                                           #IV33132
    parm="-p" 
  else
    parm="-o"
  fi

  if [[ "$2" != "cj" ]];  then
    thisCmd="agent $parm $3 start $2";                                 #IV33132
    writeCmdToStartStop "$thisCmd" "" "$1" "$4" "$startFile";          #IV33132
  fi                                                                   
                                                                       
  thisCmd="agent $parm $3 stop $2";                                    #IV33132
  writeCmdToStartStop "$thisCmd" "" "$1" "$4" "$stopFile";             #IV33132
}

function writeAgentToStartStop {
  #--------------------------------------------------------------------#IV33132
  # $1 = su userid from runas file                                     #IV33132
  # $2 = product code                                                  #IV33132
  # $3 = autostart (yes|no) from runas file... optional                #IV33132
  #--------------------------------------------------------------------#IV33132
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  getCmdWrappersInitd "$2";                                            #IV33132
                                                                       
  if [[ "$2" != "cj" ]]; then                                          #IV33132
      thisCmd="agent start $2";                                        #IV33132
      writeCmdToStartStop "$thisCmd" "" "$1" "$3" "$startFile";        #IV33132
  fi

  thisCmd="agent stop $2";                                             #IV33132
  if [[ "$2" = "kf" ]]; then                                           #IV33132
    thisCmdSrch="agent -f stop $2";                                    #IV33132
  else                                                                 #IV33132
    thisCmdSrch="";                                                    #IV33132
  fi;                                                                  #IV33132
  writeCmdToStartStop "$thisCmd" "$thisCmdSrch" "$1" "$3" "$stopFile"; #IV33132
}

function writeServerToStartStop {
  #--------------------------------------------------------------------#IV33132
  # $1 = su userid from runas file                                     #IV33132
  # $2 = TEMS name                                                     #IV33132
  # $3 = autostart (yes|no) from runas file... optional                #IV33132
  #--------------------------------------------------------------------#IV33132
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  getCmdWrappersInitd "ms";                                            #IV33132

  thisCmd="server start $2";                                           #IV33132
  writeCmdToStartStop "$thisCmd" "" "$1" "$3" "${startFile}.ms";       #IV33132

  thisCmd="server stop $2";                                            #IV33132
  writeCmdToStartStop "$thisCmd" "" "$1" "$3" "${stopFile}.ms";        #IV33132
}

function writeComponentInstance {
  #--------------------------------------------------------------------#IV97694
  # $1 = su userid from runas file                                     #IV97694
  # $2 = product code                                                  #IV97694
  # $3 = instance                                                      #IV97694
  # $4 = autostart (yes|no) from xxxrunas.cfg... optional              #IV97694
  #--------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print -- "\"$1\" \"$2\" \"$3\" \"$4\"" >> "$ComponentInstanceFile";  #IV97694
}                                        # (writeComponentInstance())  #IV97694

function deleteComponentInstance {
  #--------------------------------------------------------------------#IV97694
  # $1 = su userid from runas file                                     #IV97694
  # $2 = product code                                                  #IV97694
  # $3 = instance                                                      #IV97694
  # $4 = autostart (yes|no) from xxxrunas.cfg... optional              #IV97694
  #--------------------------------------------------------------------#IV97694
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  tmpFile=$(grep -v "\"$1\" \"$2\" \"$3\" \"$4\"" "$ComponentInstanceFile" 2> /dev/null); #IV97694
  print -- "$tmpFile" > "$ComponentInstanceFile";                      #IV97694
}                                        # (deleteComponentInstance()) #IV97694

######################################################################
######################################################################

function getValueFromLine {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print $(print $line | cut -d">" -f2 | cut -d"<" -f1)
}

function getToken1FromLine {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print $(print $line | cut -d">" -f1 | cut -d"<" -f2)
}

function getDefaultUser {
  #--------------------------------------------------------------------#
  # Get default user for product configured in kdyrunas/kcirunas
  # <productCode>mq</productCode>
  # <default>
  #   <user>defaultUser</user>
  #   <autoStart>yes</autoStart>
  # </default>
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  typeset host=$(hostname | cut -d. -f1)
  typeset cfgKdy=$CANDLEHOME/config/${host}_kdyrunas.cfg
  typeset cfgKci=$CANDLEHOME/config/kcirunas.cfg
  typeset defUser1
  [[ -f "$cfgKdy" ]] && 
    defUser1=$(getDefaultUserAndAutoStart $cfgKdy $1 | cut -d":" -f1 )
  typeset defUser2
  [[ -f "$cfgKci" ]] && 
    defUser2=$(getDefaultUserAndAutoStart $cfgKci $1| cut -d":" -f1 )
  typeset result=$defUser2
  [[ -n "$defUser1" ]] && result=$defUser1
  #print "default user is: $result"
  print $result
}

function getDefaultAutoStart {
  #--------------------------------------------------------------------#
  # Get default autostart for product configured in kdyrunas/kcirunas
  # <productCode>mq</productCode>
  # <default>
  #   <user>defaultUser</user>
  #   <autoStart>yes</autoStart>
  # </default>
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  typeset host=$(hostname | cut -d. -f1)
  typeset cfgKdy=$CANDLEHOME/config/${host}_kdyrunas.cfg
  typeset cfgKci=$CANDLEHOME/config/kcirunas.cfg
  typeset defAutostart1
  [[ -f "$cfgKdy" ]] && 
    defAutostart1=$(getDefaultUserAndAutoStart $cfgKdy $1 | cut -d":" -f2 )
  typeset defAutostart2
  [[ -f "$cfgKci" ]] && 
    defAutostart2=$(getDefaultUserAndAutoStart $cfgKci $1| cut -d":" -f2 )
  typeset result=$defAutostart2
  [[ -n "$defAutostart1" ]] && result=$defAutostart1
  print $result
}

function getDefaultUserAndAutoStart {
  #--------------------------------------------------------------------#
  # Get default user and autostart for product configured in 
  # kdyrunas/kcirunas
  # returns "default user:default Autostart"
  #
  # <default>
  #   <!user>defuseros</user>
  #   <autoStart>yes</autoStart>
  # </default>
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  typeset runasFile=$1
  typeset pc=$2
  typeset defUser
  typeset defAutoStart

  #print "parsing $runasFile for default values of $pc"
  typeset defaultFlag=false

  while read line ; do
    #print "$line"
    token1=$(getToken1FromLine)
    if [[ "$token1" = "productCode" ]]; then
      if [[ "$(getValueFromLine)" = "$pc" ]]; then
          prodFlag=true
        else
          prodFlag=false
        fi
    fi
    if [[ "$prodFlag" = true ]]; then
      case "$token1" in
        /agent)
          prodFlag=false
          ;;
        !productCode)
          prodFlag=false
          ;;
        "default")
          defaultFlag=true
          ;;
        autoStart)
          [[ "$defaultFlag" = "true" ]] && defAutoStart=$(getValueFromLine)
          ;;
        user)
          [[ "$defaultFlag" = "true" ]] && defUser=$(getValueFromLine)
          ;;
        "/default")
          defaultFalg=false
          #Writing to file default autostart and default user
          #print "Def user: $defUser, Def autostart $defAutoStart"

          break;
          ;;
      esac
    fi   #prodFlag
  done < $runasFile
  print "$defUser:$defAutoStart"
}

######################################################################
######################################################################

function grepInStart {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  if [[ -f "$startFile" ]]; then                                       #IV33132
    print $(grep "$1" "$startFile" 2> /dev/null);                      #IV33132
  else                                                                 #IV33132
    print "";                                                          #IV33132
  fi                                                                   #IV33132
}

function grepInComponentInstance {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  if [[ -f "$ComponentInstanceFile" ]]; then                           #IV33132
    print $(grep "$1" "$ComponentInstanceFile" 2> /dev/null);          #IV33132
  else                                                                 #IV33132
    print "";                                                          #IV33132
  fi                                                                   #IV33132
}

######################################################################
######################################################################

function update_rcFile {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print "$editString" > $SEDscript
  checkit=0
  rcFileTmp=$rcTmp/${rcBaseName}.new.$$
  cp -f $rcFile $rcFileTmp
  checkit=$?
  if [[ $checkit -eq 0 ]]; then
    sed -f $SEDscript $rcFileTmp > $rcFile
    checkit=$?
    if [[ $checkit -ne 0 ]]; then
      cp -f $rcFileTmp $rcFile
    fi
  fi
  rm -f $rcFileTmp
  rm -f $SEDscript
}

function update_sdTargetFile {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  print "$editString" > $SEDscript
  checkit=0
  sdTargetFileTmp=$rcTmp/${sdBase}.${sdTarget}.new.$$
  cp -f $sdTargetFile $sdTargetFileTmp
  checkit=$?
  if [[ $checkit -eq 0 ]]; then
    sed -f $SEDscript $sdTargetFileTmp > $sdTargetFile
    checkit=$?
    if [[ $checkit -ne 0 ]]; then
      cp -f $sdTargetFileTmp $sdTargetFile
    fi
  fi
  rm -f $sdTargetFileTmp
  rm -f $SEDscript
}

######################################################################
######################################################################

function checkForCfg {
  #--------------------------------------------------------------------#
  # Check for <hostname>_kdyrunas.cfg file.
  # The <hostname> is a short name only.
  # <agent>
  #  <productCode>gb</productCode>
  #  <instance>
  #   <name>instancename</name>
  #   <user>userid</user>
  #  </instance>
  #  <default>
  #   <name>instancename</name>
  #   <autoStart>userid</autoStart>
  #  </default>
  # </agent>
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  pCode=$1
  runas=$2
  tmp=0
  hostFlag=false
  prodFlag=false
  instanceFlag=false
  host=$(hostname | cut -d. -f1)
  cfgname=$CANDLEHOME/config/${host}_${runas}.cfg
  [[ "$runas" = kcirunas ]] && cfgname=$CANDLEHOME/config/${runas}.cfg

  # if file does not exist, return
  [[ ! -f "$cfgname" ]] && return
  username=""
  instance=""
  autoStart=""
  defaultAutoStart=$(getDefaultAutoStart $pCode)
  [[ -z "$defaultAutoStart" ]] && defaultAutoStart="yes"
  defaultUser=$(getDefaultUser $pCode)

  while read line ; do
    token1=$(getToken1FromLine)

    if [[ "$token1" = "productCode" ]]; then
      if [[ "$(getValueFromLine)" = "$pCode" ]]; then
        prodFlag=true
      else
        prodFlag=false
      fi
    fi

    if [[ "$prodFlag" = true ]]; then
      case "$token1" in
        /agent)
          prodFlag=false
          ;;
        !productCode)
          prodFlag=false
          ;;
        autoStart)
          autoStart=$(getValueFromLine)
          ;;
        name)
          instance=$(getValueFromLine)
          ;;
        user)
          username=$(getValueFromLine)
          ;;
        /instance)
          [[ -z "$username" && -n "$defaultUser" ]] && username="$defaultUser"
          [[ -z "$username" ]] && username=$user
          [[ -z "$autoStart" ]] && autoStart=$defaultAutoStart
          if [[ -z "$instance" || "$instance" = "None" ]]; then
            if [[ "$boot_mode" = initd ]]; then                        #IV97694
              isExist=$(grepInStart "agent start $pCode");             #IV33132
              [[ -z "$isExist" ]] && 
                writeAgentToStartStop $username $pCode $autoStart
            else
              # When checking kcirunas and kdyrunas for austostart,    #IV97694
              # we have to remove the !autostart record if it exists   #IV97694
              isExistNo=$(grepInComponentInstance "\"$username\" \"$pCode\" \"\" \"no\"");       #IV97694
              isExistYes=$(grepInComponentInstance "\"$username\" \"$pCode\" \"\" \"yes\"");     #IV97694
              isExist=$(grepInComponentInstance "\"$username\" \"$pCode\" \"\" \"$autoStart\""); #IV97694
              case "$autoStart" in                                     #IV97694
                yes)                                                   #IV97694
                  [[ -n "$isExistNo" ]] &&
                    deleteComponentInstance $username $pCode "" no;    #IV97694
                ;;                                                     #IV97694
                no)                                                    #IV97694
                  [[ -n "$isExistYes" ]] &&
                    deleteComponentInstance $username $pCode "" yes;   #IV97694
                ;;                                                     #IV97694
              esac                                                     #IV97694
              [[ -z "$isExist" ]] &&
                writeComponentInstance $username $pCode "" $autoStart; #IV97694
            fi
          else
            if [[ "$pCode" = "ms" ]]; then                             #IV40954
              if [[ "$boot_mode" = initd ]]; then                      #IV97694
                isExist=$(grepInStart "server start $instance");       #IV40954
                [[ -z "$isExist" ]] && 
                  writeServerToStartStop $username $instance $autoStart; #IV40954
              else
                # When checking kcirunas and kdyrunas for austostart,  #IV97694
                # we have to remove the !autostart record if it exists #IV97694
                isExistNo=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"no\"");       #IV97694
                isExistYes=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"yes\"");     #IV97694
                isExist=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"$autoStart\""); #IV97694
                case "$autoStart" in                                   #IV97694
                  yes)                                                 #IV97694
                    [[ -n "$isExistNo" ]] &&
                      deleteComponentInstance $username $pCode $instance no; #IV97694
                  ;;                                                   #IV97694
                  no)                                                  #IV97694
                    [[ -n "$isExistYes" ]] &&
                      deleteComponentInstance $username $pCode $instance yes; #IV97694
                  ;;                                                   #IV97694
                esac                                                   #IV97694
                [[ -z "$isExist" ]] && 
                  writeComponentInstance $username $pCode $instance $autoStart; #IV97694
              fi
            else                                                       #IV40954
              # defect 73338 - use -p for um instances
              if [[ "$pCode" = "um" ]]; then
                parm="-p"
              else
                parm="-o"
              fi
              if [[ "$boot_mode" = initd ]]; then                      #IV97694
                isExist=$(grepInStart "agent $parm $instance start $pCode" |
                  grep $username); #IV33132
                [[ -z "$isExist" ]] && 
                  writeAgentInstanceToStartStop $username $pCode $instance $autoStart
              else
                # When checking kcirunas and kdyrunas for austostart,  #IV97694
                # we have to remove the !autostart record if it exists #IV97694
                isExistNo=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"no\"");       #IV97694
                isExistYes=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"yes\"");     #IV97694
                isExist=$(grepInComponentInstance "\"$username\" \"$pCode\" \"$instance\" \"$autoStart\""); #IV97694
                case "$autoStart" in                                   #IV97694
                  yes)                                                 #IV97694
                    [[ -n "$isExistNo" ]] &&
                      deleteComponentInstance $username $pCode $instance no; #IV97694
                  ;;                                                   #IV97694
                  no)                                                  #IV97694
                    [[ -n "$isExistYes" ]] &&
                      deleteComponentInstance $username $pCode $instance yes; #IV97694
                  ;;                                                   #IV97694
                esac                                                   #IV97694
                [[ -z "$isExist" ]] && 
                  writeComponentInstance $username $pCode $instance $autoStart; #IV97694
             fi
            fi
          fi
          username=""
          instance=""
          autoStart=""
          ;;
      esac
    fi   #prodFlag

  done < $cfgname

  return
}

######################################################################
######################################################################

function verifyAgentsInitd {
  #--------------------------------------------------------------------#
  # Make sure certain agents have instance included otherwise
  # this is an error.
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  # if file does not exist, return
  [[ ! -f "$startFile" ]] && return
  while read readLine ; do
    instOpt=$(print $readLine | cut -d " " -f7)
    instPC=$(print $readLine | cut -d " " -f10)
    [[ "$instOpt" != "-o" && "$instOpt" != "-p" ]] && 
      instPC=$(print $readLine | cut -d " " -f8)

    if [[ "$instPC" = "$pCode" && "$instOpt" != +(-o|-p) ]]; then
      # get product descript
      prodLine=$(grep "^$pCode" $CANDLEHOME/registry/proddsc.tbl)
      prodLine=$(print $prodLine| cut -d "|" -f2)
      Log_report warn "$prodLine ($pCode) required start/stop with instance"
    fi
  done < $startFile
}

######################################################################
######################################################################

function verifyAgentsSystemd {
  #--------------------------------------------------------------------#
  # Make sure certain agents have instance included otherwise
  # this is an error.
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  # if file does not exist, return
  [[ ! -f "$ComponentInstanceFile" ]] && return
  while read readLine ; do
    eval instPC=$(print $readLine | cut -d " " -f2)
    eval instInst=$(print $readLine | cut -d " " -f2)

    if [[ "$instPC" = "$pCode" && -z "$instInst" ]]; then
      # get product description
      prodLine=$(grep "^$pCode" $CANDLEHOME/registry/proddsc.tbl)
      prodLine=$(print $prodLine| cut -d "|" -f2)
      Log_report warn "$prodLine ($pCode) required start/stop with instance"
    fi
  done < $ComponentInstanceFile
}

######################################################################
######################################################################

function create_start_stop_file {
  #--------------------------------------------------------------------#
  # Create two files based on current install.
  # The script will write to restart script.
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  startFile=$rcTmp/start
  stopFile=$rcTmp/stop
  pcFile=$rcTmp/pc
  userFile=$rcTmp/user
  startLine=""
  tmpTemsUser=""
  tmpTemsName=""

  [[ -a "$startFile" ]] && rm $startFile
  [[ -a "$stopFile" ]] && rm $stopFile
  [[ -a "${startFile}.ms" ]] && rm "${startFile}.ms";                  #IV33132
  [[ -a "${stopFile}.ms" ]] && rm "${stopFile}.ms";                    #IV33132
  [[ -a "$pcFile" ]] && rm $pcFile
  [[ -a "$userFile" ]] && rm -f $userFile

  $CANDLEHOME/bin/cinfo -d > $pcFile

  [[ -a "$userTmp" ]] && rm -f $userTmp
  pc=$(grep "^\"..\"" $pcFile | cut -c2,3 | sort -r )
  arch=$(ls $CANDLEHOME |grep ^$os);                                   #IV33132
  for pCode in $pc ; do
    for Arch in $arch ; do
      # here Arch is the arch, pCode is pc
      if [[ -d "$CANDLEHOME/$Arch/$pCode/bin" ]]; then
        user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/bin" | awk '{print $3}')

        if [[ "$pCode" != @(iw|ui|ue|gs|w0|ii) ]]; then                #IV33132
          instance=""

          tmpuser=$user
          inst=""
          checkForCfg $pCode "kdyrunas"
          checkForCfg $pCode "kcirunas"

          [[ "$pCode" != "ms" ]] && getInstance $pCode
          instance=""

          # ensure ud, or, oy, sa, tx, ub agents already have instance setup.
          if [[ "$pCode" = @(sa|tx|qv|gb|ph|ud|or|oy|ub) ]]; then
            verifyAgentsInitd
            continue
          fi

      [[ -f $CANDLEHOME/config/${pCode}.ini ]] && result=$(
        grep "INSTALL_AGENT_TYPE=MULTI-INSTANCE" $CANDLEHOME/config/${pCode}.ini)
          if [[ -n "$result" ]]; then
            verifyAgentsInitd
            continue
          fi

          if [[ "$pCode" = "ms" ]]; then
            cms=$(ls $CANDLEHOME/tables)
            for temsName in $cms ; do
              [[ "$temsName" = @(cicatrsq|PLACEHOLDER) ]] && continue
              temsLength=$(print $temsName| wc -c)
              temsLength=$((temsLength-4))
              oldTems=$(print $temsName | cut -c ${temsLength}-)
              [[ "$oldTems" = "_old" ]] && continue
              [[ ! -d "$CANDLEHOME/tables/$temsName/ATTRLIB" ]] && continue
              writeServerToStartStop $user $temsName "yes";            #IV33132
            done  #end cms
          else  # this is not ms,
                # so we can find agent "start pp" or "-o instance start pp"
            # make sure we preserve the default um entry               #73338
            isExist=$(grepInStart "agent start $pCode");               #IV33132
            [[ -z "$isExist" ]] && 
              writeAgentToStartStop $user $pCode "yes";                #IV33132
          fi  #end ms
        fi  #end of ui
      elif [[ "$pCode" = "kf" ]]; then                                 #IV33132
        #--------------------------------------------------------------#IV33132
        # kf is special. It doesn't have a bin directory (which is how #IV33132
        # we got here, but it does need an autostart.                  #IV33132
        #--------------------------------------------------------------#IV33132
        if [[ -f "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" ]]; then
          user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" |
                 awk '{print $3}')
          isExist=$(grepInStart "start $pCode");                       #IV33132
          [[ -z "$isExist" ]] && 
            writeAgentToStartStop $user $pCode "yes";                  #IV33132
        fi
      fi  #end of bin

    done #end arch
  done  #end pc

  #--------------------------------------------------------------------#IV33132
  # We need to start the ms component first and stop it last. To do    #IV33132
  # this, we earlier wrote its entries to separate files. Now we just  #IV33132
  # need to add them to the real start and stop files. These are later #IV33132
  # merged into the final script in reverse order. So what we're doing #IV33132
  # here looks backwards. To get ms started first, we add it to the    #IV33132
  # end of the start file. To get ms stopped last, we add it to the    #IV33132
  # beginning of the stop file.                                        #IV33132
  #--------------------------------------------------------------------#IV33132
  if [[ -a "${startFile}.ms" ]]; then                                  #IV33132
    cat "${startFile}.ms" >> "$startFile";                             #IV33132
    rm "${startFile}.ms";                                              #IV33132
  fi;                                                                  #IV33132
                                                                       #IV33132
  if [[ -a "${stopFile}.ms" ]]; then                                   #IV33132
    if [[ -a "$stopFile" ]]; then                                      #IV40953
      cat "$stopFile" >> "${stopFile}.ms";                             #IV40953
    fi;                                                                #IV40953
    mv "${stopFile}.ms" "$stopFile";                                   #IV33132
  fi;                                                                  #IV33132

  if [[ "$linux_dist" = "RedHat" ]]; then
    touch /var/lock/subsys/$rcBaseName$filenum 2> /dev/null
    print "touch /var/lock/subsys/$rcBaseName$filenum 2> /dev/null" >> $startFile
    print "rm -f /var/lock/subsys/$rcBaseName$filenum 2> /dev/null" >> $stopFile
  fi
}

function create_component_instance_file {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  ComponentInstanceFile=$rcTmp/ComponentInstance
  startLine=""
  tmpTemsUser=""
  tmpTemsName=""

  [[ -a "$ComponentInstanceFile" ]] && rm $ComponentInstanceFile
  pc=$($CANDLEHOME/bin/cinfo -d |
    grep "^\"..\"" $pcFile | cut -c2,3 | sort -r )
  arch=$(ls $CANDLEHOME |grep ^$os);
  for pCode in $pc ; do
    for Arch in $arch ; do
      # here Arch is the arch, pCode is pc
      if [[ -d "$CANDLEHOME/$Arch/$pCode/bin" ]]; then
        user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/bin" | awk '{print $3}')

        if [[ "$pCode" != @(iw|ui|ue|gs|w0|ii) ]]; then                #IV33132
          instance=""
          tmpuser=$user
          inst=""
          checkForCfg $pCode "kdyrunas"
          checkForCfg $pCode "kcirunas"

          [[ "$pCode" != "ms" ]] && getInstance $pCode
          instance=""

          [[ -f $CANDLEHOME/config/${pCode}.ini ]] && 
            result=$(grep "INSTALL_AGENT_TYPE=MULTI-INSTANCE" $CANDLEHOME/config/${pCode}.ini)
          # ensure ud, or, oy, sa, tx, ub agents already have instance setup.
          if [[ -n "$result" || "$pCode" = @(sa|tx|qv|gb|ph|ud|or|oy|ub) ]]; then
            verifyAgentsSystemd
            continue
          fi

          if [[ "$pCode" = "ms" ]]; then
            cms=$(ls $CANDLEHOME/tables)
            for temsName in $cms ; do
              [[ "$temsName" = @(cicatrsq|PLACEHOLDER|*_old) ]] && continue
              [[ ! -d "$CANDLEHOME/tables/$temsName/ATTRLIB" ]] && continue
              # We only add a record with autostart yes,               #IV97694
              # if there is not already a yes or no record             #IV97694
              isExistNo=$(grepInComponentInstance "\"$pCode\" \"$temsName\" \"no\"");   #IV97694
              isExistYes=$(grepInComponentInstance "\"$pCode\" \"$temsName\" \"yes\""); #IV97694
              [[ -z "$isExistNo$isExistYes" ]] && 
                writeComponentInstance $user $pCode $temsName "yes";   #IV97694
            done  #end cms
          else  # this is not ms,
            # We only add a record with autostart yes,                 #IV33132
            # if there is not already a yes or no record               #IV33132
            isExistNo=$(grepInComponentInstance "\"$pCode\" \"\" \"no\"");   #IV97694
            isExistYes=$(grepInComponentInstance "\"$pCode\" \"\" \"yes\""); #IV97694
            [[ -z "$isExistNo$isExistYes" ]] && 
              writeComponentInstance $user $pCode "" "yes";            #IV97694
          fi  #end ms
        fi  #end of ui
      elif [[ "${pCode}" = "kf" ]]; then                               #IV33132
        #--------------------------------------------------------------#IV33132
        # kf is special. It doesn't have a bin directory (which is how #IV33132
        # we got here, but it does need an autostart.                  #IV33132
        #--------------------------------------------------------------#IV33132
        if [[ -f "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" ]]; then
          user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" |
                 awk '{print $3}')
          # We only add a record with autostart yes,                   #IV97694
          # if there is not already a yes or no record                 #IV97694
          isExistNo=$(grepInComponentInstance "\"$pCode\" \"\" \"no\"");   #IV97694
          isExistYes=$(grepInComponentInstance "\"$pCode\" \"\" \"yes\""); #IV97694
          [[ -z "$isExistNo$isExistYes" ]] && 
            writeComponentInstance $user $pCode "" "yes";              #IV97694
        fi
      fi  #end of bin

    done #end arch
  done  #end pc
}

######################################################################
######################################################################

function delete_start_all {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
   if [[ -n "$(grep 'gent start all' $rcFile 2>/dev/null)" ]]; then
      editString="/gent start all/d"
      update_rcFile
      Log_report infoRaw "";
      if [[ $checkit -ne 0 ]]; then
         Log_report fail "Delete of agent start all record failed."
      else
         Log_report info "Delete of agent start all record successful."
      fi
   else
     return
   fi
}

function delete_stop_all {
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
   if [[ -n "$(grep 'gent stop all' "$rcFile" 2>/dev/null)" ]]; then
      editString="/gent stop all/d"
      update_rcFile
      Log_report infoRaw "";
      if [[ $checkit -ne 0 ]]; then
         Log_report fail "Delete of agent stop all record failed."
      else
         Log_report info "Delete of agent stop all record successful."
      fi
   else
      return
   fi
}

######################################################################
######################################################################

function add_start_line {
  #--------------------------------------------------------------------#
  # Add one line to script
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
   startPath=$(print $line | cut -d"\"" -f2)
   if [[ -z "$(grep  "$startPath"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/start_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      Log_report infoRaw "";
      if [[ $checkit -ne 0 ]]; then
         Log_report fail "Unable to add start agent line in $rcFile."
         Log_report infoCont "$line";
      elif [[ "$line" != @(*/var/lock/subsys*) ]]; then
         Log_report info "Add of start agent line successful."
         Log_report infoCont "$line";
      fi
   else
      Log_report info "Start agent line already exists in $rcFile."
   fi
}

function add_stop_line {
  #--------------------------------------------------------------------#
  # Add one line to script
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
   stopPath=$(print "$line" | cut -d"\"" -f2)
   if [[ -z "$(grep  "$stopPath"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/stop_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      Log_report infoRaw "";
      if [[ $checkit -ne 0 ]]; then
         Log_report fail "Unable to add stop agent line in $rcFile."
      else
         Log_report info "Add of stop agent line successful."
      fi
      Log_report infoCont "$line";
   else
      Log_report info "Stop agent line already exists in $rcFile."
   fi
}

function add_requiredBy_service {
  #--------------------------------------------------------------------#
  # Add one line to script
  #--------------------------------------------------------------------#
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
   startPath=$(print $reqline | cut -d"\"" -f2)
   if [[ -z "$(grep  "$startPath"  $sdTargetFile)" ]]; then
      print "$reqline" > $SEDline
      editString="/\[Install\]/{
r $SEDline
}"
      update_sdTargetFile
      rm -f $SEDline
      if [[ $checkit -ne 0 ]]; then
         Log_report fail "Unable to add WantedBy line in $sdTargetFile."
#      else
#         Log_report info "Add of WantedBy line successful."
      fi
   else
      Log_report info "WantedBy line already exists in $sdTargetFile."
   fi
}

######################################################################
######################################################################
######################################################################
######################################################################

                                                                       #IV95024
function clean_instance_files {
  #--------------------------------------------------------------------#IV95024
  # Clean up files from malformed instances                            #IV95024
  #--------------------------------------------------------------------#IV95024
  eval $(itmsh_trace_func "${0}" "${@}");   # enable function tracing
  cinfoInstance=$($CANDLEHOME/bin/cinfo -o | $tailCMD +7 |             #IV95024
   egrep -ve '^$|Product.*Instance|None.*\(default|<not configured>'); #IV95024
  Log_report info "cinfoInstance:\n$cinfoInstance"
  configInstance=$(                                                    #IV95024
   for line in $(cat $CANDLEHOME/config/.ConfigData/ConfigInfo |       #IV95024
   sed 's@.*/@@' | grep '_' | sed 's/.config//') ; do                  #IV95024
   CPP=$(print $line | cut -c1-2); CINST=$(print $line | cut -c4-)     #IV95024
   print $CPP $CINST ; done);                                          #IV95024
  Log_report info "configInstance:\n$configInstance"
  kppInstance=$(for file in $CANDLEHOME/config/.ConfigData/k??env ;    #IV95024
   do PP=$(print $file | sed 's@.*/@@;s@^k@@;s@env$@@') ;              #IV95024
   PLAT=$(ls $CANDLEHOME/registry/${PP}[lash]*.ver 2> /dev/null |      #IV95024
   sed 's@.*/@@;s@.ver@@'| cut -c3- );                                 #IV95024
   [[ -z "$PLAT" ]] && continue;                                       #IV95024
   kppPLAT=$(print $PLAT | tr ' ' '|');                                #IV95024
   kppINST=$(cat $CANDLEHOME/config/.ConfigData/k${PP}env |            #IV95024
   cut -f1 -d '|' |sort -u| egrep -ve "FIELDSEP|default|${kppPLAT}");  #IV95024
   for inst in $kppINST ; do print $PP $inst ; done ; done)            #IV95024
  Log_report info "kppInstance:\n$kppInstance"
  print "$cinfoInstance \n$configInstance \n$kppInstance" |            #IV95024
   awk '{print $1,$2}' | sort -u | while read PP INST junk; do         #IV95024
    [[ -z "$PP" ]] && continue;                                        #IV95024
    [[ "$PP" = ms ]] && continue;                                      #IV95024
    UPP=$(print $PP |tr 'a-z' 'A-Z');                                  #IV95024
    cleanit=no;                                                        #IV95024
    configInfo=$(cat $CANDLEHOME/config/.ConfigData/ConfigInfo |       #IV95024
     grep "${PP}_${INST}");                                            #IV95024
    PLAT=$(ls $CANDLEHOME/registry/${PP}[lash]*.ver |                  #IV95024
     sed 's@.*/@@;s@.ver@@' | cut -c3-);                               #IV95024
    kppPLAT=$(print $PLAT | tr ' ' '|');                               #IV95024
    kppenv=$(cat $CANDLEHOME/config/.ConfigData/k${PP}env |            #IV95024
     cut -f1 -d '|' | sort -u |                                        #IV95024
     egrep -ve "FIELDSEP|default|$kppPLAT" | grep "$INST");            #IV95024
    cfg1=$( cd $CANDLEHOME/config;                                     #IV95024
     ls -d *${PP}_${INST}*.cfg *${PP}_${INST}*.xml 2> /dev/null);      #IV95024
    broken=$(cd $CANDLEHOME/config ; for file in $cfg1;                #IV95024
     do ls -dL $file > /dev/null 2>&1 || print $file; done);           #IV95024
    bPat=$(print $broken | tr ' ' '|');                                #IV95024
    if [[ -n "$bPat" ]]; then                                          #IV95024
      cfg=$(print "$cfg1" | egrep -ve "$bPat");                        #IV95024
    else                                                               #IV95024
      cfg="$cfg1";                                                     #IV95024
    fi                                                                 #IV95024
    config=$(                                                          #IV95024
     ls -d $CANDLEHOME/config/*${PP}_${INST}*.config 2> /dev/null);    #IV95024
    bin=$(for binPLAT in $PLAT; do                                     #IV95024
     binPPdir=$CANDLEHOME/$binPLAT/$PP/bin                             #IV95024
     ls -d $binPPdir/*${INST}_${UPP}* 2> /dev/null;                    #IV95024
     ls -d $binPPdir/*${INST}_*_${UPP}* 2> /dev/null;                  #IV95024
     ls -d $binPPdir/*${INST}_${PP}* 2> /dev/null;                     #IV95024
     ls -d $binPPdir/*${INST}_*_${PP}* 2> /dev/null;                   #IV95024
    done);                                                             #IV95024
    hist=$(for histPLAT in $PLAT ; do                                  #IV95024
     ls -d $CANDLEHOME/$histPLAT/$PP/hist/$INST 2> /dev/null;          #IV95024
    done);                                                             #IV95024
    audit=$(                                                           #IV95024
     ls -d $CANDLEHOME/auditlogs/*${PP}_${INST}* 2> /dev/null);        #IV95024
    [[ -n "$broken" ]] && cleanit=yes;                                 #IV95024
    [[ -n "$configInfo" && -z "$config" ]] && cleanit=yes;             #IV95024
    [[ -n "$kppenv" && -z "$cfg" ]] && cleanit=yes;                    #IV95024
    if [[ "yes" = "no" ]]; then                                        #IV95024
      print "$0: PP=$PP UPP=$UPP INST=$INST" >> $logFile;              #IV95024
      print "$0: cfg1="$cfg1 >> $logFile;                              #IV95024
      print "$0: broken="$broken >> $logFile;                          #IV95024
      print "$0: cfg="$cfg >> $logFile;                                #IV95024
      print "$0: configInfo=$configInfo" >> $logFile;                  #IV95024
      print "$0: config=$config" >> $logFile;                          #IV95024
      print "$0: kppenv=$kppenv" >> $logFile;                          #IV95024
      print "$0: cleanit=$cleanit" >> $logFile;                        #IV95024
      continue;                                                        #IV95024
    fi                                                                 #IV95024
    if [[ "$cleanit" = yes ]]; then                                    #IV95024
      Log_report info "Cleaning malformed instance: $PP $INST";        #IV95024
      if [[ -n "${broken}${cfg}${config}${bin}${hist}${audit}" ]];     #IV95024
      then                                                             #IV95024
        delfiles=$(print "${broken}\n${cfg}\n${config}" | sort -u);    #IV95024
        Log_report info "Removing malformed configuration files:";     #IV95024
        for delfile in $delfiles $bin $audit ; do                      #IV95024
          Log_report infoCont "$delfile";                              #IV95024
          rm -f $delfile 2> /dev/null;                                 #IV95024
        done                                                           #IV95024
        if [[ -n "$hist" ]]; then                                      #IV95024
          Log_report infoCont "$hist";                                 #IV95024
          rm -rf $hist 2> /dev/null;                                   #IV95024
        fi                                                             #IV95024
      fi                                                               #IV95024
      dataDIR=$CANDLEHOME/config/.ConfigData;                          #IV95024
      if [[ -n "${kppenv}${configInfo}" ]]; then                       #IV95024
        Log_report info "Removing malformed configuration information:"; #IV95024
      fi                                                               #IV95024
      if [[ -n "$kppenv" ]]; then                                      #IV95024
        Log_report infoCont "$dataDIR/k${PP}env updated.";             #IV95024
        kppenvtemp=$(cat $dataDIR/k${PP}env);                          #IV95024
        print "$kppenvtemp" |                                          #IV95024
         sed "/^$INST|/d" > $dataDIR/k${PP}env;                        #IV95024
      fi                                                               #IV95024
      if [[ -n "$configInfo" ]]; then                                  #IV95024
        Log_report infoCont "$dataDIR/ConfigInfo updated." ;           #IV95024
        configInfoTemp=$(cat $dataDIR/ConfigInfo);                     #IV95024
        print "$configInfoTemp" |                                      #IV95024
         sed "/${PP}_${INST}/d" > $dataDIR/ConfigInfo;                 #IV95024
      fi                                                               #IV95024
    fi                                                                 #IV95024
  done                                                                 #IV95024
}                                                                      #IV95024

######################################################################
######################################################################
######################################################################
######################################################################


######################################################################
#   MAIN   ###########################################################
######################################################################

bindir=`pwd`/$(dirname $0)
[[ -z "$(print $0 | grep '\./')" && -z "$(print $0 | grep '^/.*')" &&
  `pwd` = '/' ]] && bindir=`pwd`$(dirname $0)
[[ -n "$(print $0 | grep '^/.*')" ]] && bindir=$(dirname $0)
PROGNAME=$(basename $0)
PROGblnk="$(print $PROGNAME | sed 's/./ /g')"

. $bindir/itmsh_trace_main;                    # embed/start tracing

typeset logFile
typeset -i filenum                               # script file number  #IZ58055
typeset -i first_time                            # 1=first time        #IZ58055

UsrCmd="$0 $*"

# to support the ITM 6.1 install.sh, support "UpdateAutoRun.sh <ch>"   #58729
CANDLEHOME=$1
while getopts :h: opt ; do
  case $opt in
    h) CANDLEHOME=$OPTARG
       CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
       ;;
    ?) Use_report
       ;;
  esac
done

[[ -n "$CANDLEHOME" ]] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
# Strip trailing / if present
CANDLEHOME=$(print $CANDLEHOME | sed 's@/*$@@')
export CANDLEHOME

NLS_REPLACE_DIR=$CANDLEHOME/bin
PATH=$PATH:$NLS_REPLACE_DIR
export PATH

#$NLS-E$ "warning"
WARNING=$(nls_replace 'KCI0535I')
#$NLS-E$ "continuing"
CONTINUING=$(nls_replace 'KCI0536I')
#$NLS-E$ "failure"
FAILURE=$(nls_replace 'KCI0537I')

[[ ! -d "$CANDLEHOME" ]] && 
  Log_report fail "$CANDLEHOME is not a valid installation."
[[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]] &&
  Log_report fail "Blanks are not permitted in CANDLEHOME: $CANDLEHOME"

rcTmp=$CANDLEHOME/tmp
SEDscript=$rcTmp/SEDscript
SEDline=$rcTmp/SEDline

# if no AutoStart and not root exit now.
myID=$(id 2> /dev/null | cut -d"=" -f2 2> /dev/null |
       cut -d"(" -f1 2> /dev/null)
[[ "$myID" = "0" ]] || 
  Log_report fail "You must be \"root\" to update the auto start script."

# secure environment when installing pre 623 images into already secured itm
if [[ "$myID" = "0" && -z "$AUTORUN_SECURE" && -n "$CDNAME" ]]; then
  SECUREFILE=$CANDLEHOME/registry/SecureMode
  if [[ -f "$SECUREFILE" ]]; then
    $CANDLEHOME/bin/itmsecure.sh autorun
  fi
fi

logFile=$CANDLEHOME/logs/UpdateAutoRun.log
[[ -f $logFile ]] && rm $logFile 2>/dev/null
touch $logFile

dynShl=$CANDLEHOME/bin/dynarch.shl
[[ -x "$dynShl" ]] || Log_report fail "Cannot execute \"$dynShl\""

archTbl=$CANDLEHOME/registry/archdsc.tbl  # archTbl is needed by dynarch.shl
[[ -r "$archTbl" ]] || Log_report fail "Cannot read \"$archTbl\""

# Determine the machine's architecture.

. $dynShl $archTbl  # dynarch.shl will determine a value for cArch.
[[ -n "$cArch" ]] || 
  Log_report fail "Could not determine the machine's architecture"
os=$(print "$cArch" |cut -c1-2)

set_linuxDist                                                          #IV97694

#--------------------------------------------------------------------#IV33132.1
# We use "grep -q...", which requires the real grep command instead  #IV33132.1
# of the nonstandard one we typically get from Solaris.              #IV33132.1
#--------------------------------------------------------------------#IV33132.1
if [[ -f /usr/xpg4/bin/grep ]]; then                                 #IV33132.1
  grepCMD="/usr/xpg4/bin/grep";                                      #IV33132.1
else                                                                 #IV33132.1
  grepCMD="grep";                                                    #IV33132.1
fi;                                                                  #IV33132.1

thisOS=$(uname -s);                         # get our platform         #
if [[ "${thisOS}" = "SunOS" ]]; then
   tailCMD="tail";                          # Solaris uses old tail    #
else
   tailCMD="tail -n";                       # standard/new tail        #
fi

#----------------------------------------------------------------------#IY96372
# Set up the value of su we are going to use.                          #IY96372
# We only want to override this if the platform is HP,                 #IY96372
# since that is the only platform having a problem right now.          #IY96372
# If su is found under the paths we want to check, we use them,        #IY96372
# otherwise not, which is no worse than right now.                     #IY96372
#----------------------------------------------------------------------#IY96372
# set the default value for su                                         #IY96372
su_value=$(which su) 2> /dev/null;                                     #IY96372
[[ -f "$su_value" ]] || su_value="su";                                 #IY96372
if [[ "$os" = hp ]]; then                                              #IY96372
  # we have an HP platform, we want to check two different files       #IY96372
  [[ -f /bin/su ]] && su_value="/bin/su";                              #IY96372
  [[ -f /usr/bin/su ]] && su_value="/usr/bin/su";                      #IY96372
fi                                                                     #IY96372

#----------------------------------------------------------------------#IV95024
# Clean up files from malformed instances                              #IV95024
#----------------------------------------------------------------------#IV95024
##clean_instance_files;      #Disable this function for now           #218905.1

#----------------------------------------------------------------------#IZ58055
# Determine the name of the initd and systemd scripts.                 #IV97694
#----------------------------------------------------------------------#IZ58055
set_rcBase;                                   # get base dir/basename  #IZ58055
set_filenum;                                  # get file number        #IV97694
rcFile="$rcBaseDir/$rcBaseName$filenum";      # script full dir/fname  #IZ58055
backRcFile="$rcTmp/$rcBaseName$filenum.bak";                           #IZ58055
if [[ "$boot_mode" = systemd ]]; then                                  #IV97694
  sdBase="$sdBaseName$filenum"                # script base fname      #IV97694
  sdTargetFile="$sdBaseDir/$sdBase.$sdTarget" # target file full fname #IV97694
  backsdTargetFile="$rcTmp/$sdBase.$sdTarget.bak";                     #IV97694
fi                                                                     #IV97694

if [[ "$boot_mode" = initd ]]; then                                    #IV97694

  #--------------------------------------------------------------------#IZ58055
  # Back up/replace the existing initd script.                         #IZ58055
  #--------------------------------------------------------------------#IZ58055
  if [[ $first_time -eq 1 || ! -f "$rcFile" ]]; then                   #IZ58055
    rm -f "$backRcFile"                         # disregard old backup #IZ58055
  elif [[ ! -r "$rcFile" ]]; then               # old script readable? #IZ58055
    Log_report fail "Can't read from file $rcFile."                    #IZ58055
  elif [[ ! -w "$rcFile" ]]; then               # old script writable? #IZ58055
    Log_report fail "Can't write to file $rcFile."                     #IZ58055
  else                                          # make a backup        #IZ58055
    cp "$rcFile" "$backRcFile" 2> /dev/null                            #IZ58055
                                                                       #IZ58055
    Log_report info     "File $rcFile was backed up to $backRcFile"    #IZ58055
    Log_report infoCont "before updates.  Please review $backRcFile"   #IZ58055
    Log_report infoCont "and make changes to $rcFile if necessary."    #IZ58055
    Log_report infoCont "Caution: $backRcFile will be replaced and"    #IZ58055
    Log_report infoCont "any custom changes will be removed when"      #IZ58055
    Log_report infoCont "$PROGNAME is run again."                      #IZ58055
  fi                                                                   #IZ58055

  #--------------------------------------------------------------------#IZ58055
  # Create a new initd script base.                                    #IZ58055
  #--------------------------------------------------------------------#IZ58055
  create_initd_script "$rcFile";               # start with new script #IZ58055

  #--------------------------------------------------------------------#IZ58055
  # Update the new initd script with current start/stop incantations.  #IZ58055
  #--------------------------------------------------------------------#IZ58055
  create_start_stop_file
  if [[ -s "$startFile" && -s "$stopFile" ]]; then
    delete_start_all
    exec 3< $startFile
    while read -u3 line ; do
      add_start_line
    done
    exec 3<&-
  
    delete_stop_all
    exec 3< $stopFile
    while read -u3 line ; do
      add_stop_line
    done
    exec 3<&-
  else
    Log_report warn "Start or Stop file is empty"
  fi

else                                                                   #IV97694

  #--------------------------------------------------------------------#IV97694
  # Backup and remove the existing initd script                        #IV97694
  # when we are creating/replacing systemd target and service scripts  #IV97694
  #--------------------------------------------------------------------#IV97694
  if [[ -f "$rcFile" ]]; then                                          #IV97694
    cp "$rcFile" "$backRcFile" 2> /dev/null                            #IV97694
    remove_initd_script_for_systemd                                    #IV97694
  fi                                                                   #IV97694

  #--------------------------------------------------------------------#IV97694
  # Back up/replace the existing systemd target and service scripts.   #IV97694
  #--------------------------------------------------------------------#IV97694
  if [[ $first_time -eq 1 || ! -f "$sdTargetFile" ]]; then             #IV97694
    rm -f "$rcTmp/$sdBase.*.bak"; # remove systemd backups             #IV97694
  elif [[ ! -r "$sdTargetFile" ]]; then # old systemd target readable? #IV97694
    Log_report fail "Can't read from file $sdTargetFile.";             #IV97694
  elif [[ ! -w "$sdTargetFile" ]]; then # old systemd target writable? #IV97694
    Log_report fail "Can't write to file $sdTargetFile.";              #IV97694
  else                                  # make systemd backups         #IV97694
    Log_report info "These files were backed up before updates."       #IV97694
    for sdFile in $sdBaseDir/$sdBase.* ; do                            #IV97694
      backsdFile="$rcTmp/$(basename $sdFile).bak";                     #IV97694
      cp "$sdFile" "$backsdFile" 2> /dev/null;                         #IV97694
      Log_report infoCont "$sdFile was backed up to $backsdFile";      #IV97694
    done                                                               #IV97694
    Log_report infoCont "Please review backed up files and make";      #IV97694
    Log_report infoCont "changes if necessary to files";               #IV97694
    Log_report infoCont "$sdBaseDir/$sdBase.'*'.";                     #IV97694
    Log_report infoCont "Caution: These files will be replaced and";   #IV97694
    Log_report infoCont "any custom changes will be removed when";     #IV97694
    Log_report infoCont "$PROGNAME is run again."                      #IV97694
  fi                                                                   #IV97694

  #--------------------------------------------------------------------#IV97694
  # Create a new systemd target script base.                           #IV97694
  # This is the start of the systemd file generations.                 #IV97694
  # We need to remove any existing target and service files for        #IV97694
  # this base so that there are no left-overs when we regenerate.      #IV97694
  #--------------------------------------------------------------------#IV97694
  if [[ -f "$sdTargetFile" ]]; then                                    #IV97694
    # Delete missing / not found systemd service units                 #IV97694
    missing=$(systemctl list-units ITMAgents1\.\* | grep not-found |   #IV97694
     cut -f1 -d ' ')                                                   #IV97694
    for unit in $MISSING ; do                                          #IV97694
      systemctl stop $unit                                             #IV97694
      systemctl reset-failed $unit                                     #IV97694
    done                                                               #IV97694
    # Delete existing systemd target and service units                 #IV97694
    sdUnits=$(print $(cd $sdBaseDir;ls $sdBase.* 2>/dev/null));        #IV97694
    cmd="echo > /dev/null 2>&1";                                       #IV97694
    for sdUnit in $sdUnits ; do                                        #IV97694
      # Disable unit                                                   #IV97694
      cmd="$cmd;$systemctlCMD disable $sdUnit";                        #IV97694
      # Delete unit file                                               #IV97694
      cmd="$cmd;rm -f $sdBaseDir/${sdUnit}";                           #IV97694
    done                                                               #IV97694
    eval "$cmd"                                                        #IV97694
    cmd="";                                                            #IV97694
  fi                                                                   #IV97694
  create_target_script "$sdTargetFile";        # start with new script #IV97694
  #service scripts are created later, one for each component instance  #IV97694

  #--------------------------------------------------------------------#IV97694
  # generate a file of each component/instance.                        #IV97694
  #--------------------------------------------------------------------#IV97694
  create_component_instance_file;                                      #IV97694

  if [[ -s "$ComponentInstanceFile" ]]; then
    #------------------------------------------------------------------#IV97694
    # Create a new systemd service script for each component/instance. #IV97694
    # Update the systemd target script for each new service script.    #IV97694
    #------------------------------------------------------------------#IV97694
    typeset PP;                                                        #IV97694
    typeset INST;                                                      #IV97694
    typeset PPINST;                                                    #IV97694
    exec 3< $ComponentInstanceFile;                                    #IV97694
    while read -u3 line ; do                                           #IV97694
      set -A PPINST -- $line;                                          #IV97694
      eval USER=${PPINST[0]};                                          #IV97694
      eval PP=${PPINST[1]};                                            #IV97694
      eval INST=${PPINST[2]};                                          #IV97694
      eval AUTOSTART=${PPINST[3]};                                     #IV97694
      [[ -z "$AUTOSTART" ]] && AUTOSTART=yes;                          #IV97694
      if [[ -z "$INST" ]]; then                                        #IV97694
        PPINST=$PP;                                                    #IV97694
      else                                                             #IV97694
        PPINST=$PP.$INST;                                              #IV97694
      fi                                                               #IV97694
      # service file full fname                                        #IV97694
      sdServiceFile="$sdBaseDir/$sdBase.$PPINST.$sdService";           #IV97694
      MSGprefix="AUTOSTART:$AUTOSTART USER:$USER PP:$PP";              #IV97694
      Log_report infoRaw "";                                           #IV97694
      if [[ -n "$INST" ]]; then                                        #IV97694
        Log_report info "Processing: $MSGprefix INST:$INST";           #IV97694
      else                                                             #IV97694
        Log_report info "Processing: $MSGprefix";                      #IV97694
      fi                                                               #IV97694
      create_service_script $sdServiceFile $line;                      #IV97694
      reqline="WantedBy=$sdBase.$PPINST.$sdService";                   #IV97694
      add_requiredBy_service;                                          #IV97694
      if [[ "$AUTOSTART" = no ]]; then                                 #IV97694
        # Disable service file when autostart is no                    #IV97694
        Log_report info "Disabling $(basename $sdServiceFile)";        #IV97694
        $systemctlCMD disable $(basename $sdServiceFile);              #IV97694
      else                                                             #IV97694
        # Enable service file                                          #IV97694
        Log_report info "Enabling $(basename $sdServiceFile)";         #IV97694
        $systemctlCMD enable $(basename $sdServiceFile);               #IV97694
      fi                                                               #IV97694
    done                                                               #IV97694
    exec 3<&-;                                                         #IV97694
    # Enable target file now that all updates are complete             #IV97694
    Log_report infoRaw "";                                             #IV97694
    Log_report info "Enabling $(basename $sdTargetFile)";              #IV97694
    $systemctlCMD enable $(basename $sdTargetFile);                    #IV97694
    Log_report infoRaw "";                                             #IV97694
    Log_report info "Status of $sdBase units:";                        #IV97694
    $systemctlCMD list-unit-files --all | grep $sdBase |               #IV97694
      while read line ; do                                             #IV97694
        Log_report infoCont "$line";                                   #IV97694
      done                                                             #IV97694
  else                                                                 #IV97694
    Log_report warn "Component Instance file is empty";                #IV97694
  fi                                                                   #IV97694
fi                                                                     #IV97694

#----------------------------------------------------------------------#IZ58055
# Tell the operating system about our shiny new init script.           #IZ58055
#----------------------------------------------------------------------#IZ58055
set_runlevels;                                # chkconfig, mkitab, etc #IZ58055
                                                                       #IV23322
#$NLS-E$ "Automatic start at system initialization has been configured"#IV23322
nls_replace 'KCI0117I';                                                #IV23322
if [[ "$os" != "ai" ]]; then                                           #IV23322
  #$NLS-E$ "Automatic stop at system shutdown has been configured."    #IV23322
  nls_replace 'KCI0118I';                                              #IV23322
else                                                                   #IV23322
  #$NLS-E$ "You must manually configure automatic stop at system       #IV23322
  #$NLS-E$ shutdown. Create or modify /etc/rc.shutdown to invoke the   #IV23322
  #$NLS-E$ command '$1 stop'."                                         #IV23322
  nls_replace 'KCI0119I' "$rcFile";                                    #IV23322
fi                                                                     #IV23322
######################################################################
#   END OF MAIN   ####################################################
######################################################################
