#!/bin/ksh
# **********************************************************************
# * Licensed Materials - Property of IBM
# * 5724-C04
# * Copyright IBM Corporation 2015 All Rights Reserved
# **********************************************************************
[[ $# -ne 2 ]] && print "Usage: $0 <itm installation directory> <product code>" && exit 1

CANDLEHOME=$1
PC=$2

[[ ! -d "$CANDLEHOME" ]] && print "Invalid ITM installation directory - $1" && exit 1

sourceEnv_files="";                         # assume no override files  #

#-----------------------------------------------------------------------#
# Check for IBM-generated global overrides.                             #
#-----------------------------------------------------------------------#
sourceEnv_gibm="${CANDLEHOME}/config/global.ibm_environment";
if [[ -a "${sourceEnv_gibm}" ]]; then 
   sourceEnv_files="${sourceEnv_files} \"${sourceEnv_gibm}\"";
fi;

#-----------------------------------------------------------------------#
# Check for IBM-generated overrides.                                    #
#-----------------------------------------------------------------------#
sourceEnv_ibm="${CANDLEHOME}/config/${PC}.ibm_environment";
if [[ -a "${sourceEnv_ibm}" ]]; then 
   sourceEnv_files="${sourceEnv_files} \"${sourceEnv_ibm}\"";
fi;

#-----------------------------------------------------------------------#
# Check for customer-provided global overrides.                         #
#-----------------------------------------------------------------------#
sourceEnv_gcust="${CANDLEHOME}/config/global.environment";
if [[ -a "${sourceEnv_gcust}" ]]; then 
   sourceEnv_files="${sourceEnv_files} \"${sourceEnv_gcust}\"";
fi;

#-----------------------------------------------------------------------#
# Check for customer-provided overrides.                                #
#-----------------------------------------------------------------------#
sourceEnv_cust="${CANDLEHOME}/config/${PC}.environment";
if [[ -a "${sourceEnv_cust}" ]]; then 
   sourceEnv_files="${sourceEnv_files} \"${sourceEnv_cust}\"";
fi;

#-----------------------------------------------------------------------#
# If we have any overrides, source them into our process. We use sed to #
# prepend the export command and add quotes to the value. So, we expect #
# the source file contains 'X=Y' and we translate to 'export X="Y"'.    #
#-----------------------------------------------------------------------#
if [[ -n "${sourceEnv_files}" ]]; then
   sourceEnv_temp="${CANDLEHOME}/config/${PC}.temp_environment";
   sourceEnv_sed="s/^[A-Za-z].*$/export &/";
   sourceEnv_sed="${sourceEnv_sed};s/\(^export.*\)=\(.*\)/\1=\"\2\"/";

   ( # Locate system commands regardles of the PATH in the PP.ini file #IV80194
   export PATH=$PATH:/bin:/usr/bin
   eval "cat ${sourceEnv_files}" | sed "${sourceEnv_sed}" > "${sourceEnv_temp}";
   )                                                                   #IV80194

   . "${sourceEnv_temp}";

   rc=${?};

   ( # Locate system commands regardles of the PATH in the PP.ini file #IV80194
   export PATH=$PATH:/bin:/usr/bin
   [[ -z "${KEEP_STARTUP_FILES}" ]] && rm -f "${sourceEnv_temp}" 2>/dev/null
   )                                                                   #IV80194
else
   rc=0;
fi;

#-----------------------------------------------------------------------#
# We are sourced in ourselves. So, if we fail and exit, we're stopping  #
# the parent that called us, too.                                       #
#-----------------------------------------------------------------------#
[[ ${rc} -ne 0 ]] && exit ${rc};
